@@ -18,6 +18,10 @@
 #'                  \strong{Note:} "parquet" format is supported by the \code{arrow} package and it will need to be installed to utilise the "parquet" format.
 #' @param compress \code{FALSE | TRUE} To determine if to compress file.type. If file type is ["csv", "tsv"] then "gzip" compression is used, for file type "parquet" 
 #'                 "snappy" compression is used.
+#' @param max.batch Split the data frame by max number of rows i.e. 100,000 so that multiple files can be uploaded into AWS S3. By default when compression
+#'                  is set to \code{TRUE} and file.type is "csv" or "tsv" max.batch will split data.frame into 20 batches. This is to help the 
+#'                  performance of AWS Athena when working with files compressed in "gzip" format. \code{max.batch} will not split the data.frame 
+#'                  when loading file in parquet format. For more information please go to \href{https://github.com/DyfanJones/RAthena/issues/36}{link}
 #' @inheritParams DBI::sqlCreateTable
 #' @return \code{dbWriteTable()} returns \code{TRUE}, invisibly. If the table exists, and both append and overwrite
 #'         arguments are unset, or append = TRUE and the data frame with the new data has different column names,
@@ -73,7 +77,7 @@ Athena_write_table <-
   function(conn, name, value, overwrite=FALSE, append=FALSE,
            row.names = NA, field.types = NULL, 
            partition = NULL, s3.location = NULL, file.type = c("csv", "tsv", "parquet"),
-           compress = FALSE, ...) {
+           compress = FALSE, max.batch = Inf,...) {
     # variable checks
     stopifnot(is.character(name),
               is.data.frame(value),
@@ -82,12 +86,17 @@ Athena_write_table <-
               is.null(s3.location) || is.s3_uri(s3.location),
               is.null(partition) || is.character(partition) || is.list(partition),
               is.logical(compress))
-
+    
     sapply(tolower(names(partition)), function(x){if(x %in% tolower(names(value))){
       stop("partition ", x, " is a variable in data.frame ", deparse(substitute(value)), call. = FALSE)}})
     
     file.type = match.arg(file.type)
     
+    if(max.batch < 0) stop("`max.batch` has to be greater than 0", call. = F)
+    
+    if(!is.infinite(max.batch) && file.type == "parquet") message("Info: parquet format is splittable and AWS Athena can read parquet format ",
+                                                                  "in parrel. `max.batch` is used for compressed `gzip` format which is not splittable.")
+    
     # use default s3_staging directory is s3.location isn't provided
     if (is.null(s3.location)) s3.location <- conn@info$s3_staging
     
@@ -105,17 +114,17 @@ Athena_write_table <-
 
     if(append && is.null(partition)) stop("Athena requires the table to be partitioned to append", call. = FALSE)
 
-    t <- tempfile()
-    on.exit({unlink(t)
-             if(!is.null(conn@info$expiration)) time_check(conn@info$expiration)})
-
+    # return original Athena Types
+    if(is.null(field.types)) field.types <- dbDataType(conn, value)
     value <- sqlData(conn, value, row.names = row.names)
     
-    # compress file
-    FileLocation <- paste(t, Compress(file.type, compress), sep =".")
+    
 
     # check if arrow is installed before attempting to create parquet
     if(file.type == "parquet"){
+      # compress file
+      FileLocation <- paste(t, Compress(file.type, compress), sep =".")
+      
       if(!requireNamespace("arrow", quietly=TRUE))
         stop("The package arrow is required for R to utilise Apache Arrow to create parquet files.", call. = FALSE)
       else {
@@ -124,9 +133,17 @@ Athena_write_table <-
     }
     
     # writes out csv/tsv, uses data.table for extra speed
-    switch(file.type,
-           "csv" = data.table::fwrite(value, FileLocation, showProgress = F),
-           "tsv" = data.table::fwrite(value, FileLocation, sep = "\t", showProgress = F))
+    if (file.type == "csv"){
+      FileLocation <- split_data(value, max.batch = max.batch, compress = compress, file.type = file.type)
+    }
+    
+    if (file.type == "tsv"){
+      FileLocation <- split_data(value, max.batch = max.batch, compress = compress, file.type = file.type, sep = "\t")
+    }
+    
+    # switch(file.type,
+    #        "csv" = data.table::fwrite(value, FileLocation, showProgress = F),
+    #        "tsv" = data.table::fwrite(value, FileLocation, sep = "\t", showProgress = F))
 
     found <- dbExistsTable(conn, Name)
     if (found && !overwrite && !append) {
@@ -143,7 +160,7 @@ Athena_write_table <-
     }
 
     # send data over to s3 bucket
-    upload_data(conn, FileLocation, name, partition, s3.location, file.type, compress)
+    s3_uri <- upload_data(conn, FileLocation, name, partition, s3.location, file.type, compress)
     
     if (!append) {
       sql <- sqlCreateTable(conn, table = Name, fields = value, field.types = field.types, 
@@ -159,6 +176,23 @@ Athena_write_table <-
     res <- dbExecute(conn, paste0("MSCK REPAIR TABLE ", Name))
     dbClearResult(res)
     
+    
+    # Warn users to check if table that has been overwrite has been correctly overwritten.
+    if(compress && overwrite && file.type %in% c("csv", "tsv")){
+      s3_bucket <- split_s3_uri(s3.location)$bucket
+      s3.location <- gsub("/$", "", s3.location)
+      if(grepl(name, s3.location)){s3.location <- gsub(paste0("/", name,"$"), "", s3.location)}
+      s3.location <- paste0("'",s3.location,"/", name,"/'")
+      
+      message("Info: Please check that all files in ", s3.location," have been replaces with:\n",
+              paste0("s3://",s3_bucket, "/", s3_uri, collapse = "\n"))
+    }
+    
+    # t <- tempfile()
+    on.exit({lapply(FileLocation, unlink)
+      #unlink(t)
+      if(!is.null(conn@info$expiration)) time_check(conn@info$expiration)})
+    
     invisible(TRUE)
   }
 
@@ -171,7 +205,7 @@ upload_data <- function(con, x, name, partition = NULL, s3.location= NULL,  file
   
   # s3_file name
   FileType <- if(compress) Compress(file.type, compress) else file.type
-  FileName <- paste(name, FileType, sep = ".")
+  FileName <- paste(if (length(x) > 1) paste0(name,"_", 1:length(x)) else name, FileType, sep = ".")
   
   # s3 bucket and key split
   s3_info <- split_s3_uri(s3.location)
@@ -189,10 +223,10 @@ upload_data <- function(con, x, name, partition = NULL, s3.location= NULL,  file
   
   tryCatch(s3 <- con@ptr$resource("s3"),
            error = function(e) py_error(e))
-  tryCatch(s3$Bucket(s3_info$bucket)$upload_file(Filename = x, Key = s3_key),
-           error = function(e) py_error(e))
-  
-  invisible(TRUE)
+  for (i in 1:length(x)){
+    tryCatch(s3$Bucket(s3_info$bucket)$upload_file(Filename = x[i], Key = s3_key[i]),
+             error = function(e) py_error(e))}
+  s3_key
 }
 
 #' @rdname AthenaWriteTables
@@ -203,11 +237,11 @@ setMethod(
   function(conn, name, value, overwrite=FALSE, append=FALSE,
            row.names = NA, field.types = NULL, 
            partition = NULL, s3.location = NULL, file.type = c("csv", "tsv", "parquet"),
-           compress = FALSE, ...){
+           compress = FALSE, max.batch = Inf, ...){
     if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
     Athena_write_table(conn, name, value, overwrite, append,
                       row.names, field.types,
-                      partition, s3.location, file.type, compress)
+                      partition, s3.location, file.type, compress, max.batch)
     })
 
 #' @rdname AthenaWriteTables
@@ -217,11 +251,11 @@ setMethod(
   function(conn, name, value, overwrite=FALSE, append=FALSE,
            row.names = NA, field.types = NULL, 
            partition = NULL, s3.location = NULL, file.type = c("csv", "tsv", "parquet"),
-           compress = FALSE, ...){
+           compress = FALSE, max.batch = Inf, ...){
     if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
     Athena_write_table(conn, name, value, overwrite, append,
                       row.names, field.types,
-                      partition, s3.location, file.type, compress)
+                      partition, s3.location, file.type, compress, max.batch)
   })
 
 #' @rdname AthenaWriteTables
@@ -231,11 +265,11 @@ setMethod(
   function(conn, name, value, overwrite=FALSE, append=FALSE,
            row.names = NA, field.types = NULL, 
            partition = NULL, s3.location = NULL, file.type = c("csv", "tsv", "parquet"),
-           compress = FALSE, ...){
+           compress = FALSE, max.batch = Inf,...){
     if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
     Athena_write_table(conn, name, value, overwrite, append,
                       row.names, field.types,
-                      partition, s3.location, file.type, compress)
+                      partition, s3.location, file.type, compress, max.batch)
   })
 
 #' Converts data frame into suitable format to be uploaded to Athena
@@ -252,17 +286,27 @@ NULL
 #' @export
 setMethod("sqlData", "AthenaConnection", function(con, value, row.names = NA, ...) {
   stopifnot(is.data.frame(value))
-  value <- sqlRownamesToColumn(value, row.names)
+  
+  Value <- copy(value)
+  Value <- sqlRownamesToColumn(Value, row.names)
   field_names <- tolower(gsub("\\.", "_", make.names(names(value), unique = TRUE)))
-  DIFF <- setdiff(field_names, names(value))
-  names(value) <- field_names
+  DIFF <- setdiff(field_names, names(Value))
+  names(Value) <- field_names
   if (length(DIFF) > 0) message("Info: data.frame colnames have been converted to align with Athena DDL naming convertions: \n",paste0(DIFF, collapse= ",\n"))
-  for(i in seq_along(value)){
-    if(is.list(value[[i]])){
-      value[[i]] <- sapply(value[[i]], paste, collapse = "|")
-    }
-  }
-  value
+  
+  # get R col types
+  col_types <- sapply(Value, class)
+  
+  # preprosing proxict format
+  posixct_cols<- names(Value)[sapply(col_types, function(x) "POSIXct" %in% x)]
+  # create timestamp in athena format: https://docs.aws.amazon.com/athena/latest/ug/data-types.html
+  for (col in posixct_cols) set(Value, j=col, value=strftime(Value[[col]], format="%Y-%m-%d %H:%M:%OS3"))
+  
+  # preprosing list format
+  list_cols <- names(Value)[sapply(col_types, function(x) "list" %in% x)]
+  for (col in list_cols) set(Value, j=col, value=sapply(Value[[col]], paste, collapse = "|"))
+  
+  Value
 })
 
 #' Creates query to create a simple Athena table
@@ -329,6 +373,7 @@ setMethod("sqlCreateTable", "AthenaConnection",
     if (is.null(s3.location)) s3.location <- con@info$s3_staging
 
     table1 <- gsub(".*\\.", "", table)
+    table <- paste0(quote_identifier(con,  unlist(strsplit(table,"\\."))), collapse = ".")
     
     s3.location <- gsub("/$", "", s3.location)
     if(grepl(table1, s3.location)){s3.location <- gsub(paste0("/", table1,"$"), "", s3.location)}
@@ -350,12 +395,15 @@ createFields <- function(con, fields, field.types) {
     fields <- vapply(fields, function(x) DBI::dbDataType(con, x), character(1))
   }
   if (!is.null(field.types)) {
+    names(field.types) <- tolower(gsub("\\.", "_", make.names(names(field.types), unique = TRUE)))
     fields[names(field.types)] <- field.types
   }
   
   field_names <- tolower(gsub("\\.", "_", make.names(names(fields), unique = TRUE)))
   DIFF <- setdiff(field_names, names(fields))
   if (length(DIFF) > 0) message("Info: data.frame colnames have been converted to align with Athena DDL naming convertions: \n",paste0(DIFF, collapse= ",\n"))
+  
+  field_names <- quote_identifier(con, field_names)
   field.types <- unname(fields)
   paste0(field_names, " ", field.types)
 }
@@ -394,3 +442,17 @@ Compress <- function(file.type, compress){
          "parquet" = paste("snappy", file.type, sep = "."))
   } else {file.type}
 }
+
+# helper function to format column and table name
+quote_identifier <- function(conn, x, ...) {
+  if (length(x) == 0L) {
+    return(DBI::SQL(character()))
+  }
+  if (any(is.na(x))) {
+    stop("Cannot pass NA to sqlCreateTable()", call. = FALSE)
+  }
+  if (nzchar(conn@quote)) {
+    x <- gsub(conn@quote, paste0(conn@quote, conn@quote), x, fixed = TRUE)
+  }
+  DBI::SQL(paste(conn@quote, encodeString(x), conn@quote, sep = ""))
+}
